@@ -1,6 +1,6 @@
 package Tickit::Widget::Table::HeaderCell;
 {
-  $Tickit::Widget::Table::HeaderCell::VERSION = '0.003';
+  $Tickit::Widget::Table::HeaderCell::VERSION = '0.100';
 }
 use strict;
 use warnings;
@@ -12,7 +12,7 @@ Tickit::Widget::Table::HeaderCell - header cell for a table
 
 =head1 VERSION
 
-version 0.003
+version 0.100
 
 =head1 SYNOPSIS
 
@@ -20,26 +20,28 @@ version 0.003
 
 =cut
 
+use Tickit::Utils qw(textwidth);
+
+use Tickit::Style;
+
+BEGIN {
+	style_definition base =>
+		fg => 'white',
+		bg => 'blue',
+		b => 1,
+		spacing => 0;
+}
+
 =head1 METHODS
 
 =cut
 
-=head2 render
+=head2 new
 
-Display the label using the appropriate spacing and attributes
+Instantiate a new header cell.
 
 =cut
 
-sub render {
-	my $self = shift;
-	my $win = $self->window or return;
-	my $txt = $self->text;
-	# TODO should we be doing something with this?
-	# my $x = $self->display_xpos;
-	$win->goto(0,0);
-	$win->print($txt . (' ' x (-$self->table->padding + $self->cols - length $txt)), bg => 4, fg => 7, b => 1);
-}
-
 sub new {
 	my $class = shift;
 	my %args = @_;
